// to_dos.cpp

// g++ -o to_dos to_dos.cpp

// Converts an UNIX style text file into a MS-DOS style one by
//  adding ^M characters at the end of each line immediately before the ^J,
//  and adds a ^Z character at the end of the file.
//  (Note: ^J = LF = 0x0A; ^M = CR = 0x0D; ^Z = 0x1A)
//
// Note: the program tries to be smart about not adding extra ^M and ^Z
//  characters if they're already there, i.e., CRLF won't become CRCRLF, etc.
//
// Zero, one or two arguments can be given. If zero are given, to_dos
// uses stdin for input and stdout for output. If one is given, it's used
// for stdin, and stdout gets the output. If two are given, the first is
// used for input, the second for output.


using namespace std;

#include <iostream.h>
#include <fstream.h>
#include <string.h>


// Arbitrary buffer size, just enough so that we're able to do buffered
// processing instead of character-by-character
#define BUF_SIZE (4096)


// Main program loop
int main(int argc, char *argv[])
{
  istream *infile = NULL;
  ostream *outfile = NULL;
  
  char c, in_buf[BUF_SIZE];
  // Output buffer could grow to 2x size of input buffer
  char out_buf[BUF_SIZE*2];
  
  int run, tmp, dest;
  
  // Check how many arguments are passed, determine what input/output files
  //  we will be using
  switch(argc)
  {
    // None: look for input from stdin, and dump output to stdout
    //  (i.e., act as a filter)
    case 1:
      infile = (istream *) &cin;
      outfile = (ostream *) &cout;
      // Nothing to do here...
      break;
  
    // One: File given is input, output to stdout
    case 2:
      infile = new ifstream(argv[1]);
      outfile = (ostream *) &cout;
      break;

    // Two: Input from first one, output to second one
    case 3:
      // Make sure that we aren't trying to open the same file for
      //  input and output...
      if(strcmp(argv[1], argv[2]) == 0)
      {
        cerr << "to_dos: Input and output files must have different names. Aborting." << endl;
        return 0; // Exit early
      }
      infile = new ifstream(argv[1]);
      outfile = new ofstream(argv[2]);
      break;

    // Incorrect number of paramaters, complain and exit
    default:
      cerr << "to_dos usage:  to_dos [<UNIX filename> [<MS-DOS filename>]]" << endl;
      return 0; // Exit early
      break;
  }
  

  // Check to see if we have valid infile and outfile streams -- if not, complain
  //  and abort.
  if(!infile)
  {
    cerr << "to_dos: Unable to open input file, aborting." << endl;
    return 0; // Exit early
  }
  if(!outfile)
  {
    cerr << "to_dos: Unable to open output file, aborting." << endl;
    return 0; // Exit early
  }


  // Preload carry-over byte in input buffer
  infile->get(c);
  in_buf[0] = c;
  
  // If this is the only byte in the file and it's a LF, stuff a CR in before it
  //  and a ^Z after it
  if( (infile->eof()) && (c == 0x0A) )
  {
    out_buf[0] = 0x0D;
    out_buf[1] = c;
    out_buf[2] = 0x1A;
    dest = 3;    
  }
  else
  {
    out_buf[0] = c;
    dest = 1;

    // If the char isn't a ^Z, stuff it in there
    if(c != 0x1A)
    {
      out_buf[1] = 0x1A;
      dest++;
    }
  }
  
  // Main loop for processing file
  while(infile->good())
  {
    // Read in another chunk
    infile->read(in_buf+1, BUF_SIZE-2);
    run = infile->gcount();

    // This is necessary to make sure that we don't process any leftover junk
    //  in the buffer between blocks of data
    in_buf[run+1] = 0x00;

    // Run through it, copy each chunk that's a line to the output
    tmp = 0;
    dest = 0;
    while(tmp < run)
    {
      out_buf[dest++] = in_buf[tmp];
      
      // Insert extra CRs as needed
      if( (in_buf[tmp] != 0x0D) && (in_buf[tmp+1] == 0x0A) )
      {
        out_buf[dest++] = 0x0D;
      }

      tmp++;
    }
    
    // End-of-file situation
    if(infile->eof())
    {
      out_buf[dest++] = in_buf[tmp];

      // Add a ^Z if it's not already there
      if(in_buf[tmp] != 0x1A)
      {
        out_buf[dest++] = 0x1A;
      }      
    }
    // Otherwise, preserve last character for next run
    else
    {
      in_buf[0] = in_buf[tmp];
    }
    
    // Write out line
    outfile->write(out_buf, dest);
  }

  return 0; // Exit, done
}


