// to_unix.cpp

// g++ -o to_unix to_unix.cpp

// Converts an MS-DOS style text file into a UNIX style one by
//  stripping the ^M character at the end of each line, and the ^Z
//  character at the end of the file.
//  (Note: ^J = LF = 0x0A; ^M = CR = 0x0D; ^Z = 0x1A)
//
// Note: to_unix tries to be smart enough to only convert CRLF into LF, and not
//  simply strip every CR it encounters in the source file.
//
// Zero, one or two arguments can be given. If zero are given, to_unix
// uses stdin for input and stdout for output. If one is given, it's used
// for stdin, and stdout gets the output. If two are given, the first is
// used for input, the second for output.


using namespace std;

#include <iostream.h>
#include <fstream.h>


// Arbitrary buffer size -- enough so that we're doing our work on buffered
//  input instead of character-by-character
#define BUF_SIZE (4096)


// Main program loop
int main(int argc, char *argv[])
{
  istream *infile = NULL;
  ostream *outfile = NULL;
  
  char c, in_buf[BUF_SIZE], out_buf[BUF_SIZE];
  int run, tmp, dest;
    
  // Check how many arguments are passed, determine what input/output files
  //  we will be using
  switch(argc)
  {
    // None: look for input from stdin, and dump output to stdout
    //  (i.e., act as a filter)
    case 1:
      infile = (istream *) &cin;
      outfile = (ostream *) &cout;
      // Nothing to do here...
      break;
  
    // One: File given is input, output to stdout
    case 2:
      infile = new ifstream(argv[1]);
      outfile = (ostream *) &cout;
      break;

    // Two: Input from first one, output to second one
    case 3:
      if(strcmp(argv[1], argv[2]) == 0)
      {
        cerr << "to_unix: Input and output files must have different names. Aborting." << endl;
        return 0; // Exit early
      }
      infile = new ifstream(argv[1]);
      outfile = new ofstream(argv[2]);
      break;

    // Incorrect number of paramaters, complain and exit
    default:
      cerr << "to_unix usage:  to_unix [<MS-DOS filename> [<UNIX filename>]]" << endl;
      return 0; // Exit early
      break;
  }
  

  // Check to see if we have valid infile and outfile streams -- if not, complain
  //  and abort.
  if(!infile)
  {
    cerr << "to_unix: Unable to open input file, aborting." << endl;
    return 0; // Exit early
  }
  if(!outfile)
  {
    cerr << "to_unix: Unable to open output file, aborting." << endl;
    return 0; // Exit early
  }


  // Preload carry-over byte in input buffer
  infile->get(c);
  in_buf[0] = c;
  
  // If this is the only byte in the file and it's a ^Z, chuck it; setup
  //  destination buffer for write, since we're going to fall-through the
  //  main processing loop
  if( (infile->eof()) && (c != 0x1A) )
  {
    out_buf[0] = c;
    dest = 1;    
  }
  else
  {
    dest = 0;
  }
  
  // Main loop for processing file
  while(infile->good())
  {
    // Read in another chunk
    infile->read(in_buf+1, BUF_SIZE-1);
    run = infile->gcount();

    // Run through it, copy each chunk that's a line to the output
    tmp = 0;
    dest = 0;
    while(tmp < run)
    {
      if( (!((in_buf[tmp] == 0x0D) && (in_buf[tmp+1] == 0x0A))) ) 
      {
        out_buf[dest++] = in_buf[tmp];
      }

      tmp++;
    }
    
    // End-of-file situation
    if(infile->eof())
    {
      // Throw away ^Z if we're at the end of the file
      if(in_buf[tmp] != 0x1A)
      {
        out_buf[dest++] = in_buf[tmp];
      }      
    }
    // Otherwise, preserve last character for next run
    else
    {
      in_buf[0] = in_buf[tmp];
    }
    
    // Write out line
    outfile->write(out_buf, dest);
  }
  
  return 0; // Exit, done
}


